@@ -1,4 +1,4 @@
-/*  
+/*
  * eepro100.c -- This file implements the eepro100 driver for etherboot.
  *
  *
@@ -8,9 +8,9 @@
  *
  * AW Computer Systems is contributing to the free software community
  * by paying for this driver and then putting the result under GPL.
- * 
+ *
  * If you need a Linux device driver, please contact BitWizard for a
- * quote. 
+ * quote.
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -36,28 +36,26 @@
  *              Jul 25 1997  V1.00  REW  Tested by AW to work in a PROM
  *                                       Cleanup for publication
  *
- * This is the etherboot-3.1 compatible intel etherexpress Pro/100B 
- * driver. 
- * 
+ * This is the etherboot intel etherexpress Pro/100B driver.
+ *
  * It was written from scratch, with Donald Beckers eepro100.c kernel
  * driver as a guideline. Mostly the 82557 related definitions and the
  * lower level routines have been cut-and-pasted into this source.
- * 
+ *
  * The driver was finished before Intel got the NDA out of the closet.
- * I still don't have the docs. 
+ * I still don't have the docs.
  * */
 
-
-
 /* Philosophy of this driver.
- * 
- * Probing: 
  *
- * Using a subset of "bios32" and "pci" functions of the linux kernel,
- * the pci 82557 chip is detected.
+ * Probing:
  *
- * 
- * Initialization: 
+ * Using the pci.c functions of the Etherboot code, the 82557 chip is detected.
+ * It is verified that the BIOS initialized everything properly and if
+ * something is missing it is done now.
+ *
+ *
+ * Initialization:
  *
  *
  * The chip is then initialized to "know" its ethernet address, and to
@@ -74,12 +72,12 @@
  * Transmitting and recieving.
  *
  * We have only one transmit descriptor. It has two buffer descriptors:
- * one for the header, and the other for the data. 
+ * one for the header, and the other for the data.
  * We have only one receive buffer. The chip is told to recieve packets,
  * and suspend itself once it got one. The recieve (poll) routine simply
  * looks at the recieve buffer to see if there is already a packet there.
- * if there is, the buffer is copied, and the reciever is restarted. 
- * 
+ * if there is, the buffer is copied, and the reciever is restarted.
+ *
  * Caveats:
  *
  * The etherboot framework moves the code to the 32k segment from
@@ -88,25 +86,22 @@
  * features, this will overflow.
  * The number under "hex" in the output of size that scrolls by while
  * compiling should be less than 8000. Maybe even the stack is up there,
- * so that you need even more headroom. 
- * 
- * If you run into trouble, the method used to give "pci.c" dynamic
- * allocation should be used to allocate the larger variables (like
- * the packet buffers)
- * */
-
-#ifdef INCLUDE_EEPRO100
-
+ * so that you need even more headroom.
+ */
 
 /* The etherboot authors seem to dislike the argument ordering in
  * outb macros that Linux uses. I disklike the confusion that this
  * has caused even more.... This file uses the Linux argument ordering.  */
-#define LINUX_OUT_MACROS
+/* Sorry not us. It's inherted code from FreeBSD. [The authors] */
 
-#include "netboot.h"
+#include "etherboot.h"
 #include "nic.h"
-#include <linux/bios32.h>
-#include <linux/pci.h>
+#include "pci.h"
+#include "cards.h"
+#include "timer.h"
+
+#undef	virt_to_bus
+#define	virt_to_bus(x)	((unsigned long)x)
 
 static int ioaddr;
 
@@ -117,7 +112,6 @@ typedef   signed short s16;
 typedef unsigned int   u32;
 typedef   signed int   s32;
 
-
 enum speedo_offsets {
   SCBStatus = 0, SCBCmd = 2,      /* Rx/Command Unit command and status. */
   SCBPointer = 4,                 /* General purpose pointer. */
@@ -127,11 +121,8 @@ enum speedo_offsets {
   SCBEarlyRx = 20,                /* Early receive byte count. */
 };
 
-
-static int read_eeprom(int ioaddr, int location);
-static void udelay (int val);
-void hd (void *where, int n);
-
+static int do_eeprom_cmd(int cmd, int cmd_len);
+void hd(void *where, int n);
 
 /***********************************************************************/
 /*                       I82557 related defines                        */
@@ -143,22 +134,15 @@ void hd (void *where, int n);
 #define EE_SHIFT_CLK    0x01    /* EEPROM shift clock. */
 #define EE_CS           0x02    /* EEPROM chip select. */
 #define EE_DATA_WRITE   0x04    /* EEPROM chip data in. */
-#define EE_WRITE_0      0x01
-#define EE_WRITE_1      0x05
 #define EE_DATA_READ    0x08    /* EEPROM chip data out. */
+#define EE_WRITE_0      0x4802
+#define EE_WRITE_1      0x4806
 #define EE_ENB          (0x4800 | EE_CS)
 
-
-/* Delay between EEPROM clock transitions.
-   This is a "nasty" timing loop, but PC compatible machines are defined
-   to delay an ISA compatible period for the SLOW_DOWN_IO macro.  */
-#define eeprom_delay(nanosec)   do { int _i = 3; while (--_i > 0) \
-                                     { __SLOW_DOWN_IO; }} while (0)
+#define udelay(n)       waiton_timer2(((n)*TICKS_PER_MS)/1000)
 
 /* The EEPROM commands include the alway-set leading bit. */
-#define EE_WRITE_CMD    (5 << 6)
-#define EE_READ_CMD     (6 << 6)
-#define EE_ERASE_CMD    (7 << 6)
+#define EE_READ_CMD     6
 
 /* The SCB accepts the following controls for the Tx and Rx units: */
 #define  CU_START       0x0010
@@ -179,17 +163,15 @@ void hd (void *where, int n);
 enum phy_chips { NonSuchPhy=0, I82553AB, I82553C, I82503, DP83840, S80C240,
                                          S80C24, PhyUndefined, DP83840A=10, };
 
-
-
 /* Commands that can be put in a command list entry. */
 enum commands {
-  CmdNOp = 0, 
-  CmdIASetup = 1, 
-  CmdConfigure = 2, 
+  CmdNOp = 0,
+  CmdIASetup = 1,
+  CmdConfigure = 2,
   CmdMulticastList = 3,
-  CmdTx = 4, 
-  CmdTDR = 5, 
-  CmdDump = 6, 
+  CmdTx = 4,
+  CmdTDR = 5,
+  CmdDump = 6,
   CmdDiagnose = 7,
 
   /* And some extra flags: */
@@ -198,7 +180,6 @@ enum commands {
   CmdTxFlex = 0x0008,       /* Use "Flexible mode" for CmdTx command. */
 };
 
-
 /* How to wait for the command unit to accept a command.
    Typically this takes 0 ticks. */
 static inline void wait_for_cmd_done(int cmd_ioaddr)
@@ -208,9 +189,8 @@ static inline void wait_for_cmd_done(int cmd_ioaddr)
   while(inb(cmd_ioaddr) && --wait >= 0);
 }
 
-
 /* Elements of the dump_statistics block. This block must be lword aligned. */
-struct speedo_stats {
+static struct speedo_stats {
         u32 tx_good_frames;
         u32 tx_coll16_errs;
         u32 tx_late_colls;
@@ -230,9 +210,8 @@ struct speedo_stats {
         u32 done_marker;
 } lstats;
 
-
 /* A speedo3 TX buffer descriptor with two buffers... */
-struct TxFD {
+static struct TxFD {
   volatile s16 status;
   s16 command;
   u32 link;          /* void * */
@@ -243,11 +222,8 @@ struct TxFD {
   s32 tx_buf_size0;  /* Length of Tx hdr. */
   u32 tx_buf_addr1;  /* void *, data to be transmitted.  */
   s32 tx_buf_size1;  /* Length of Tx data. */
-} txfd; 
+} txfd;
 
-
-
-/* The Speedo3 Rx buffer descriptors. */
 struct RxFD {               /* Receive frame descriptor. */
   volatile s16 status;
   s16 command;
@@ -256,9 +232,15 @@ struct RxFD {               /* Receive frame descriptor. */
   u16 count;
   u16 size;
   char packet[1518];
-} rxfd;
-
+};
 
+#ifdef	USE_LOWMEM_BUFFER
+#define rxfd ((struct RxFD *)(0x10000 - sizeof(struct RxFD)))
+#define ACCESS(x) x->
+#else
+static struct RxFD rxfd;
+#define ACCESS(x) x.
+#endif
 
 static int congenb = 0;         /* Enable congestion control in the DP83840. */
 static int txfifo = 8;          /* Tx FIFO threshold in 4 byte units, 0-15 */
@@ -266,55 +248,33 @@ static int rxfifo = 8;          /* Rx FIFO threshold, default 32 bytes. */
 static int txdmacount = 0;      /* Tx DMA burst length, 0-127, default 0. */
 static int rxdmacount = 0;      /* Rx DMA length, 0 means no preemption. */
 
-
-
 /* I don't understand a byte in this structure. It was copied from the
  * Linux kernel initialization for the eepro100. -- REW */
-struct ConfCmd {
+static struct ConfCmd {
   s16 status;
   s16 command;
   u32 link;
   unsigned char data[22];
 } confcmd = {
   0, CmdConfigure,
-  (u32) & txfd, 
+  (u32) & txfd,
   {22, 0x08, 0, 0,  0, 0x80, 0x32, 0x03,  1, /* 1=Use MII  0=Use AUI */
    0, 0x2E, 0,  0x60, 0,
    0xf2, 0x48,   0, 0x40, 0xf2, 0x80,        /* 0x40=Force full-duplex */
    0x3f, 0x05, }
 };
 
-
-
-#define TIME_OUT 1000000
-
-/* The "pci" code needs to allocate a few structures. It wants to
- * increment the "kernel memory ends here" pointer to allocate memory.
- * I guess that a machine with PCI has more than 2Mb of memory, so
- * that is where those things are put. Those structures don't survive
- * the "jump to kernel start".
- *   */
-#define MEM_START 0x200000   /* Memory starts at 2Mb for now....  */
-#define MEM_END   0
-
-static unsigned short eeprom [0x40];
-
-static void calibrate_delay (void);
-
-
 /***********************************************************************/
 /*                       Locally used functions                        */
 /***********************************************************************/
 
-
 /* Support function: mdio_write
  *
- * This probably writes to the "physical media interface chip". 
+ * This probably writes to the "physical media interface chip".
  * -- REW
  */
 
-
-static int mdio_write(int ioaddr, int phy_id, int location, int value)
+static int mdio_write(int phy_id, int location, int value)
 {
   int val, boguscnt = 64*4;         /* <64 usec. to complete, typ 27 ticks */
 
@@ -325,19 +285,18 @@ static int mdio_write(int ioaddr, int phy_id, int location, int value)
 
     val = inl(ioaddr + SCBCtrlMDI);
     if (--boguscnt < 0) {
-      printf(" mdio_write() timed out with val = %8.8x.\n", val);
+      printf(" mdio_write() timed out with val = %X.\n", val);
     }
   } while (! (val & 0x10000000));
   return val & 0xffff;
 }
 
-
 /* Support function: mdio_read
  *
- * This probably reads a register in the "physical media interface chip". 
+ * This probably reads a register in the "physical media interface chip".
  * -- REW
  */
-static int mdio_read(int ioaddr, int phy_id, int location)
+static int mdio_read(int phy_id, int location)
 {
   int val, boguscnt = 64*4;               /* <64 usec. to complete, typ 27 ticks */
   outl(0x08000000 | (location<<16) | (phy_id<<21), ioaddr + SCBCtrlMDI);
@@ -346,75 +305,54 @@ static int mdio_read(int ioaddr, int phy_id, int location)
 
     val = inl(ioaddr + SCBCtrlMDI);
     if (--boguscnt < 0) {
-      printf( " mdio_read() timed out with val = %8.8x.\n", val);
+      printf( " mdio_read() timed out with val = %X.\n", val);
     }
   } while (! (val & 0x10000000));
   return val & 0xffff;
 }
 
-
-/* Support function: read_eeprom
- * reads a value from the eeprom at a specified location.
- * Arguments: ioaddr:    address of the 82557 chip
- *            location:  address of the location to read from the eeprom.
- * returns:   value read from eeprom at location.
- */
-static int read_eeprom(int ioaddr, int location)
+/* The fixes for the code were kindly provided by Dragan Stancevic
+   <visitor@valinux.com> to strictly follow Intel specifications of EEPROM
+   access timing.
+   The publicly available sheet 64486302 (sec. 3.1) specifies 1us access
+   interval for serial EEPROM.  However, it looks like that there is an
+   additional requirement dictating larger udelay's in the code below.
+   2000/05/24  SAW */
+static int do_eeprom_cmd(int cmd, int cmd_len)
 {
-        int i;
-        unsigned short retval = 0;
-        int ee_addr = ioaddr + SCBeeprom;
-        int read_cmd = location | EE_READ_CMD;
-        
-        outw(EE_ENB & ~EE_CS, ee_addr);
-        outw(EE_ENB, ee_addr);
-        
-        /* Shift the read command bits out. */
-        for (i = 10; i >= 0; i--) {
-                short dataval = (read_cmd & (1 << i)) ? EE_DATA_WRITE : 0;
-                outw(EE_ENB | dataval, ee_addr);
-                eeprom_delay(100);
-                outw(EE_ENB | dataval | EE_SHIFT_CLK, ee_addr);
-                eeprom_delay(150);
-                outw(EE_ENB | dataval, ee_addr);        /* Finish EEPROM a clock tick. */
-                eeprom_delay(250);
-        }
-        outw(EE_ENB, ee_addr);
-        
-        for (i = 15; i >= 0; i--) {
-                outw(EE_ENB | EE_SHIFT_CLK, ee_addr);
-                eeprom_delay(100);
-                retval = (retval << 1) | ((inw(ee_addr) & EE_DATA_READ) ? 1 : 0);
-                outw(EE_ENB, ee_addr);
-                eeprom_delay(100);
-        }
+	unsigned retval = 0;
+	long ee_addr = ioaddr + SCBeeprom;
 
-        /* Terminate the EEPROM access. */
-        outw(EE_ENB & ~EE_CS, ee_addr);
-        return retval;
+	outw(EE_ENB, ee_addr); udelay(2);
+	outw(EE_ENB | EE_SHIFT_CLK, ee_addr); udelay(2);
+
+	/* Shift the command bits out. */
+	do {
+		short dataval = (cmd & (1 << cmd_len)) ? EE_WRITE_1 : EE_WRITE_0;
+		outw(dataval, ee_addr); udelay(2);
+		outw(dataval | EE_SHIFT_CLK, ee_addr); udelay(2);
+		retval = (retval << 1) | ((inw(ee_addr) & EE_DATA_READ) ? 1 : 0);
+	} while (--cmd_len >= 0);
+	outw(EE_ENB, ee_addr); udelay(2);
+
+	/* Terminate the EEPROM access. */
+	outw(EE_ENB & ~EE_CS, ee_addr);
+	return retval;
 }
 
-
-static void inline whereami (char *str)
+static inline void whereami (const char *str)
 {
-#if 0
-  printf ("%s\r\n", str);
+#if	0
+  printf ("%s\n", str);
   sleep (2);
 #endif
 }
 
-
-
-/***********************************************************************/
-/*                    Externally visible functions                     */
-/***********************************************************************/
-
-
-/* function: eepro100_reset / eth_reset
- * resets the card. This is used to allow Linux to probe the card again
- * from a "virginal" state.... 
+/* function: eepro100_reset
+ * resets the card. This is used to allow Etherboot to probe the card again
+ * from a "virginal" state....
  * Arguments: none
- *            
+ *
  * returns:   void.
  */
 
@@ -423,9 +361,7 @@ static void eepro100_reset(struct nic *nic)
   outl(0, ioaddr + SCBPort);
 }
 
-
-
-/* function: eepro100_transmit / eth_transmit
+/* function: eepro100_transmit
  * This transmits a packet.
  *
  * Arguments: char d[6]:          destination ethernet address.
@@ -435,29 +371,28 @@ static void eepro100_reset(struct nic *nic)
  * returns:   void.
  */
 
-static void eepro100_transmit(struct nic *nic, char *d, unsigned int t, unsigned int s, char *p)
+static void eepro100_transmit(struct nic *nic, const char *d, unsigned int t, unsigned int s, const char *p)
 {
   struct eth_hdr {
-    unsigned char dst_addr[6];
-    unsigned char src_addr[6];
+    unsigned char dst_addr[ETH_ALEN];
+    unsigned char src_addr[ETH_ALEN];
     unsigned short type;
   } hdr;
   unsigned short status;
   int to;
   int s1, s2;
-  
+
   status = inw(ioaddr + SCBStatus);
   /* Acknowledge all of the current interrupt sources ASAP. */
   outw(status & 0xfc00, ioaddr + SCBStatus);
-  
-#ifdef DEBUG
-  printf ("transmitting type %x packet (%d bytes). status = %x, cmd=%x\r\n", 
+
+#ifdef	DEBUG
+  printf ("transmitting type %hX packet (%d bytes). status = %hX, cmd=%hX\n",
 	  t, s, status, inw (ioaddr + SCBCmd));
 #endif
 
-
-  memcpy (&hdr.dst_addr, d, 6);
-  memcpy (&hdr.src_addr, &arptable[ARP_CLIENT].node, 6);
+  memcpy (&hdr.dst_addr, d, ETH_ALEN);
+  memcpy (&hdr.src_addr, nic->node_addr, ETH_ALEN);
 
   hdr.type = htons (t);
 
@@ -473,8 +408,8 @@ static void eepro100_transmit(struct nic *nic, char *d, unsigned int t, unsigned
   txfd.tx_buf_addr1 = virt_to_bus (p);
   txfd.tx_buf_size1 = s;
 
-#ifdef DEBUG
-  printf ("txfd: \r\n");
+#ifdef	DEBUG
+  printf ("txfd: \n");
   hd (&txfd, sizeof (txfd));
 #endif
 
@@ -483,24 +418,22 @@ static void eepro100_transmit(struct nic *nic, char *d, unsigned int t, unsigned
   wait_for_cmd_done(ioaddr + SCBCmd);
 
   s1 = inw (ioaddr + SCBStatus);
-  to = TIME_OUT;
-  while (!txfd.status && --to)
+  load_timer2(10*TICKS_PER_MS);		/* timeout 10 ms for transmit */
+  while (!txfd.status && timer2_running())
     /* Wait */;
   s2 = inw (ioaddr + SCBStatus);
 
-#ifdef DEBUG 
-  printf ("Tx: Loop executed %d times.\r\n", TIME_OUT-to);
-  printf ("s1 = %x, s2 = %x.\r\n", s1, s2);
+#ifdef	DEBUG
+  printf ("s1 = %hX, s2 = %hX.\n", s1, s2);
 #endif
 }
 
-
 /* function: eepro100_poll / eth_poll
- * This recieves a packet from the network. 
+ * This recieves a packet from the network.
  *
  * Arguments: none
- *            
- * returns:   1 if a packet was recieved. 
+ *
+ * returns:   1 if a packet was recieved.
  *            0 if no pacet was recieved.
  * side effects:
  *            returns the packet in the array nic->packet.
@@ -509,76 +442,85 @@ static void eepro100_transmit(struct nic *nic, char *d, unsigned int t, unsigned
 
 static int eepro100_poll(struct nic *nic)
 {
-  int to;
-
-  to = TIME_OUT;
-  while (!rxfd.status && --to)
-    /* Wait */;
+  if (!ACCESS(rxfd)status)
+    return 0;
 
   /* Ok. We got a packet. Now restart the reciever.... */
-  rxfd.status = 0;
-  rxfd.command = 0xc000;
-  outl(virt_to_bus(&rxfd), ioaddr + SCBPointer);
+  ACCESS(rxfd)status = 0;
+  ACCESS(rxfd)command = 0xc000;
+  outl(virt_to_bus(&(ACCESS(rxfd)status)), ioaddr + SCBPointer);
   outw(INT_MASK | RX_START, ioaddr + SCBCmd);
   wait_for_cmd_done(ioaddr + SCBCmd);
 
-  if (to) {
-#ifdef DEBUG
-    printf ("Got a packet: Len = %d.\r\n", rxfd.count & 0x3fff);
+#ifdef	DEBUG
+  printf ("Got a packet: Len = %d.\n", ACCESS(rxfd)count & 0x3fff);
 #endif
-    nic->packetlen =  rxfd.count & 0x3fff;
-    bcopy (rxfd.packet, nic->packet, sizeof (rxfd.packet));
-#ifdef DEBUG
-    hd (nic->packet, 0x30);
+  nic->packetlen =  ACCESS(rxfd)count & 0x3fff;
+  memcpy (nic->packet, ACCESS(rxfd)packet, nic->packetlen);
+#ifdef	DEBUG
+  hd (nic->packet, 0x30);
 #endif
-    return 1;
-  } else
-    return 0;
+  return 1;
 }
 
 static void eepro100_disable(struct nic *nic)
 {
+    /* See if this PartialReset solves the problem with interfering with
+       kernel operation after Etherboot hands over. - Ken 20001102 */
+    outl(2, ioaddr + SCBPort);
 }
 
 /* exported function: eepro100_probe / eth_probe
  * initializes a card
- *            
- * side effects: 
+ *
+ * side effects:
  *            leaves the ioaddress of the 82557 chip in the variable ioaddr.
  *            leaves the 82557 initialized, and ready to recieve packets.
  */
 
-struct nic *eepro100_probe(struct nic *nic, unsigned short *probeaddrs)
+struct nic *eepro100_probe(struct nic *nic, unsigned short *probeaddrs, struct pci_device *p)
 {
-  int pci_index;
-  u16 sum = 0;
-  int i, j, to;
-  unsigned short value;
-  int options;
-  int promisc;
+	unsigned short sum = 0;
+	int i;
+	int read_cmd, ee_size;
+	unsigned short value;
+	int options;
+	int promisc;
 
-  if (probeaddrs == 0 || probeaddrs[0] == 0)
-          return 0;
+	/* we cache only the first few words of the EEPROM data
+	   be careful not to access beyond this array */
+	unsigned short eeprom[16];
 
-  ioaddr = probeaddrs[0] & ~3; /* Mask the bit that says "this is an io addr" */
+	if (probeaddrs == 0 || probeaddrs[0] == 0)
+		return 0;
+	ioaddr = probeaddrs[0] & ~3; /* Mask the bit that says "this is an io addr" */
 
-  /* Ok. Got one. Read the eeprom. */
-  for (j = 0, i = 0; i < 0x40; i++) {
-	value = read_eeprom(ioaddr, i);
-	eeprom[i] = value;
-	sum += value;
+	adjust_pci_device(p);
+
+	if ((do_eeprom_cmd(EE_READ_CMD << 24, 27) & 0xffe0000)
+		== 0xffe0000) {
+		ee_size = 0x100;
+		read_cmd = EE_READ_CMD << 24;
+	} else {
+		ee_size = 0x40;
+		read_cmd = EE_READ_CMD << 22;
+	}
+
+	for (i = 0, sum = 0; i < ee_size; i++) {
+		unsigned short value = do_eeprom_cmd(read_cmd | (i << 16), 27);
+		if (i < (int)(sizeof(eeprom)/sizeof(eeprom[0])))
+			eeprom[i] = value;
+		sum += value;
+	}
+
+  for (i=0;i<ETH_ALEN;i++) {
+	nic->node_addr[i] =  (eeprom[i/2] >> (8*(i&1))) & 0xff;
   }
-  
-  printf ("Ethernet addr: ");
-  for (i=0;i<6;i++) {
-	arptable[ARP_CLIENT].node[i] =  (eeprom[i/2] >> (8*(i&1))) & 0xff;
-	printf ("%b%c", arptable[ARP_CLIENT].node[i] , i < 5?':':' '); 
-  }
-  printf ("\r\n");
+  printf ("Ethernet addr: %!\n", nic->node_addr);
 
   if (sum != 0xBABA)
-	printf("eepro100: Invalid EEPROM checksum %#4.4x, "
-	       "check settings before activating this device!\r\n", sum);
+	printf("eepro100: Invalid EEPROM checksum %#hX, "
+	       "check settings before activating this device!\n", sum);
   outl(0, ioaddr + SCBPort);
   udelay (10);
 
@@ -587,37 +529,36 @@ struct nic *eepro100_probe(struct nic *nic, unsigned short *probeaddrs)
   outl(virt_to_bus(&lstats), ioaddr + SCBPointer);
   outw(INT_MASK | CU_STATSADDR, ioaddr + SCBCmd);
   wait_for_cmd_done(ioaddr + SCBCmd);
-  
+
   whereami ("set stats addr.");
   /* INIT RX stuff. */
-  
+
   /* Base = 0 */
   outl(0, ioaddr + SCBPointer);
   outw(INT_MASK | RX_ADDR_LOAD, ioaddr + SCBCmd);
   wait_for_cmd_done(ioaddr + SCBCmd);
 
   whereami ("set rx base addr.");
-  
-  rxfd.status  = 0x0001;
-  rxfd.command = 0x0000;
-  rxfd.link    = virt_to_bus(&rxfd);
-  rxfd.rx_buf_addr = (int) &nic->packet;
-  rxfd.count   = 0;
-  rxfd.size    = 1528;
-  
-  outl(virt_to_bus(&rxfd), ioaddr + SCBPointer);
+
+  ACCESS(rxfd)status  = 0x0001;
+  ACCESS(rxfd)command = 0x0000;
+  ACCESS(rxfd)link    = virt_to_bus(&(ACCESS(rxfd)status));
+  ACCESS(rxfd)rx_buf_addr = (int) &nic->packet;
+  ACCESS(rxfd)count   = 0;
+  ACCESS(rxfd)size    = 1528;
+
+  outl(virt_to_bus(&(ACCESS(rxfd)status)), ioaddr + SCBPointer);
   outw(INT_MASK | RX_START, ioaddr + SCBCmd);
   wait_for_cmd_done(ioaddr + SCBCmd);
 
   whereami ("started RX process.");
 
   /* Start the reciever.... */
-  rxfd.status = 0;
-  rxfd.command = 0xc000;
-  outl(virt_to_bus(&rxfd), ioaddr + SCBPointer);
+  ACCESS(rxfd)status = 0;
+  ACCESS(rxfd)command = 0xc000;
+  outl(virt_to_bus(&(ACCESS(rxfd)status)), ioaddr + SCBPointer);
   outw(INT_MASK | RX_START, ioaddr + SCBCmd);
 
-  
   /* INIT TX stuff. */
 
   /* Base = 0 */
@@ -630,36 +571,35 @@ struct nic *eepro100_probe(struct nic *nic, unsigned short *probeaddrs)
   txfd.command      = (CmdIASetup);
   txfd.status       = 0x0000;
   txfd.link         = virt_to_bus (&confcmd);
-  
-  { 
-	char *p = (char *)&txfd.tx_desc_addr;
 
-	for (i=0;i<6;i++)
-	p[i] = arptable[ARP_CLIENT].node[i];
+  {
+	char *t = (char *)&txfd.tx_desc_addr;
+
+	for (i=0;i<ETH_ALEN;i++)
+		t[i] = nic->node_addr[i];
   }
-  
-#ifdef DEBUG
-  printf ("Setup_eaddr:\r\n");
+
+#ifdef	DEBUG
+  printf ("Setup_eaddr:\n");
   hd (&txfd, 0x20);
 #endif
   /*      options = 0x40; */ /* 10mbps half duplex... */
   options = 0x00;            /* Autosense */
 
   promisc = 0;
-  
-  printf ("eeprom[6] = %x \r\n", eeprom[6]);
+
   if (   ((eeprom[6]>>8) & 0x3f) == DP83840
 	  || ((eeprom[6]>>8) & 0x3f) == DP83840A) {
-	int mdi_reg23 = mdio_read(ioaddr, eeprom[6] & 0x1f, 23) | 0x0422;
+	int mdi_reg23 = mdio_read(eeprom[6] & 0x1f, 23) | 0x0422;
 	if (congenb)
 	  mdi_reg23 |= 0x0100;
-	printf("  DP83840 specific setup, setting register 23 to %x.\r\n",
+	printf("  DP83840 specific setup, setting register 23 to %hX.\n",
 	       mdi_reg23);
-	mdio_write(ioaddr, eeprom[6] & 0x1f, 23, mdi_reg23);
+	mdio_write(eeprom[6] & 0x1f, 23, mdi_reg23);
   }
   whereami ("Done DP8340 special setup.");
   if (options != 0) {
-	mdio_write(ioaddr, eeprom[6] & 0x1f, 0,
+	mdio_write(eeprom[6] & 0x1f, 0,
 		   ((options & 0x20) ? 0x2000 : 0) |    /* 100mbps? */
 		   ((options & 0x10) ? 0x0100 : 0)); /* Full duplex? */
 	whereami ("set mdio_register.");
@@ -681,13 +621,10 @@ struct nic *eepro100_probe(struct nic *nic, unsigned short *probeaddrs)
 
   whereami ("started TX thingy (config, iasetup).");
 
-  to = TIME_OUT;
-  while (!txfd.status && --to)
+  load_timer2(10*TICKS_PER_MS);
+  while (!txfd.status && timer2_running())
 	/* Wait */;
 
-#ifdef DEBUG 
-  printf ("\r\nLoop executed %d times.\r\n", TIME_OUT-to);
-#endif
   nic->reset = eepro100_reset;
   nic->poll = eepro100_poll;
   nic->transmit = eepro100_transmit;
@@ -695,91 +632,9 @@ struct nic *eepro100_probe(struct nic *nic, unsigned short *probeaddrs)
   return nic;
 }
 
-static int loops_per_sec;
-static int loops_per_usec;
-
-static void udelay (int val)
-{
-  int c;
-
-  for(c=0; c<val*loops_per_usec; c++) {
-    /* Nothing */;
-  }
-}
-
-
-#if 1
-
-/* I don't have the possibility to test the below code. Murphey tells me
- * it won't work.
- */
-
-static void calibrate_delay (void)
-{
-  loops_per_usec = 500;
-}
-#else
-
-
-#define __delay udelay
-#define jiffies currticks()
-#define HZ 18
-
-#define LPS_PREC 8
-
-/* Copied from /usr/src/linux/init/main.c */
-/* with just a few modifications. */
-void calibrate_delay (void)
-{
-  int ticks;
-  int loopbit;
-  int lps_precision = LPS_PREC;
-
-  loops_per_sec = (1<<12);
-  loops_per_usec = 1;
-
-  printk("Calibrating delay loop.. ");
-  while (loops_per_sec <<= 1) {
-    /* wait for "start of" clock tick */
-    ticks = jiffies;
-    while (ticks == jiffies)
-      /* nothing */;
-    /* Go .. */
-    ticks = jiffies;
-    __delay(loops_per_sec);
-    ticks = jiffies - ticks;
-    if (ticks)
-      break;
-  }
-
-  /* Do a binary approximation to get loops_per_second set to equal one clock
-     (up to lps_precision bits) */
-  loops_per_sec >>= 1;
-  loopbit = loops_per_sec;
-  while ( lps_precision-- && (loopbit >>= 1) ) {
-    loops_per_sec |= loopbit;
-    ticks = jiffies;
-    while (ticks == jiffies);
-    ticks = jiffies;
-    __delay(loops_per_sec);
-    if (jiffies != ticks)   /* longer than 1 tick */
-      loops_per_sec &= ~loopbit;
-  }
-
-  /* finally, adjust loops per second in terms of seconds instead of clocks */    
-  loops_per_sec *= HZ;
-  /* Round the value and print it */      
-  printk("ok - %d.%d BogoMIPS\n",
-	 (loops_per_sec+25000)/500000,
-	 ((loops_per_sec+25000)/50000) % 10);
-  loops_per_usec = loops_per_sec / 1000000;
-}
-#endif
-
-
 /*********************************************************************/
 
-#ifdef DEBUG
+#ifdef	DEBUG
 
 /* Hexdump a number of bytes from memory... */
 void hd (void *where, int n)
@@ -789,12 +644,11 @@ void hd (void *where, int n)
   while (n > 0) {
     printf ("%X ", where);
     for (i=0;i < ( (n>16)?16:n);i++)
-      printf (" %b", ((char *)where)[i]);
-    printf ("\n\r");
+      printf (" %hhX", ((char *)where)[i]);
+    printf ("\n");
     n -= 16;
     where += 16;
   }
 }
 #endif
 
-#endif /* INCLUDE_EEPRO100 */
