@@ -1,8 +1,7 @@
 /* char_io.c - basic console input and output */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 1996  Erich Boleyn  <erich@uruk.org>
- *  Copyright (C) 1999, 2000  Free Software Foundation, Inc.
+ *  Copyright (C) 1999, 2000, 2001  Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +20,10 @@
 
 #include <shared.h>
 
+#ifdef SUPPORT_HERCULES
+# include <hercules.h>
+#endif
+
 #ifdef SUPPORT_SERIAL
 # include <serial.h>
 #endif
@@ -302,7 +305,11 @@ get_cmdline (char *prompt, char *cmdline, int maxlen,
 	{
 	  xpos -= count;
 
-	  if (terminal & TERMINAL_CONSOLE)
+	  if ((terminal & TERMINAL_CONSOLE)
+# ifdef SUPPORT_HERCULES
+	      || (terminal & TERMINAL_HERCULES)
+# endif /* SUPPORT_HERCULES */
+	      )
 	    {
 	      int y = getxy () & 0xFF;
 	      
@@ -336,7 +343,11 @@ get_cmdline (char *prompt, char *cmdline, int maxlen,
 	{
 	  xpos += count;
 
-	  if (terminal & TERMINAL_CONSOLE)
+	  if ((terminal & TERMINAL_CONSOLE)
+# ifdef SUPPORT_HERCULES
+	      || (terminal & TERMINAL_HERCULES)
+# endif /* SUPPORT_HERCULES */
+	      )
 	    {
 	      int y = getxy () & 0xFF;
 	      
@@ -374,7 +385,8 @@ get_cmdline (char *prompt, char *cmdline, int maxlen,
       if (full)
 	{
 	  /* Recompute the section number.  */
-	  if (lpos + plen < CMDLINE_WIDTH)
+	  if ((terminal & TERMINAL_DUMB)
+	      || (lpos + plen < CMDLINE_WIDTH))
 	    section = 0;
 	  else
 	    section = ((lpos + plen - CMDLINE_WIDTH)
@@ -437,44 +449,51 @@ get_cmdline (char *prompt, char *cmdline, int maxlen,
 
 	  pos++;
 	}
-
-      /* Fill up the rest of the line with spaces.  */
-      for (; i < start + len; i++)
+      
+      if (! (terminal & TERMINAL_DUMB))
 	{
-	  grub_putchar (' ');
-	  pos++;
-	}
-
-      /* If the cursor is at the last position, put `>' or a space,
-	 depending on if there are more characters in BUF.  */
-      if (pos == CMDLINE_WIDTH)
-	{
-	  if (start + len < llen)
-	    grub_putchar ('>');
-	  else
-	    grub_putchar (' ');
+         /* Fill up the rest of the line with spaces.  */
+         for (; i < start + len; i++)
+           {
+             grub_putchar (' ');
+             pos++;
+           }
 	  
-	  pos++;
-	}
-
-      /* Back to XPOS.  */
-      if (terminal & TERMINAL_CONSOLE)
-	{
-	  int y = getxy () & 0xFF;
-	  
-	  gotoxy (xpos, y);
-	}
+         /* If the cursor is at the last position, put `>' or a space,
+            depending on if there are more characters in BUF.  */
+         if (pos == CMDLINE_WIDTH)
+           {
+             if (start + len < llen)
+               grub_putchar ('>');
+             else
+               grub_putchar (' ');
+             
+             pos++;
+           }
+         
+         /* Back to XPOS.  */
+         if ((terminal & TERMINAL_CONSOLE)
+# ifdef SUPPORT_HERCULES
+             || (terminal & TERMINAL_HERCULES)
+# endif /* SUPPORT_HERCULES */
+             )
+           {
+             int y = getxy () & 0xFF;
+             
+             gotoxy (xpos, y);
+           }
 # ifdef SUPPORT_SERIAL      
-      else if (! (terminal & TERMINAL_SERIAL) && (pos - xpos > 4))
-	{
-	  grub_printf ("\e[%dD", pos - xpos);
-	}
-      else
-	{
-	  for (i = 0; i < pos - xpos; i++)
-	    grub_putchar ('\b');
-	}
+         else if (! (terminal & TERMINAL_SERIAL) && (pos - xpos > 4))
+           {
+             grub_printf ("\e[%dD", pos - xpos);
+           }
+         else
+           {
+             for (i = 0; i < pos - xpos; i++)
+               grub_putchar ('\b');
+           }
 # endif /* SUPPORT_SERIAL */
+       }
     }
 
   /* Initialize the command-line.  */
@@ -999,7 +1018,15 @@ grub_strncat (char *s1, const char *s2, int n)
 
   return 1;
 }
+#endif /* ! STAGE1_5 */
 
+/* XXX: This below is an evil hack. Certainly, we should change the
+   strategy to determine what should be defined and what shouldn't be
+   defined for each image. For example, it would be better to create
+   a static library supporting minimal standard C functions and link
+   each image with the library. Complicated things should be left to
+   computer, definitely. -okuji  */
+#if ! defined(STAGE1_5) || defined(FSYS_VSTAFS)
 int
 grub_strcmp (const char *s1, const char *s2)
 {
@@ -1015,14 +1042,20 @@ grub_strcmp (const char *s1, const char *s2)
 
   return 0;
 }
+#endif /* ! STAGE1_5 || FSYS_VSTAFS */
 
+#ifndef STAGE1_5
 /* Wait for a keypress and return its code.  */
 int
 getkey (void)
 {
   int c = -1;
   
-  if (terminal & TERMINAL_CONSOLE)
+  if ((terminal & TERMINAL_CONSOLE)
+#ifdef SUPPORT_HERCULES
+      || (terminal & TERMINAL_HERCULES)
+#endif /* SUPPORT_HERCULES */
+      )
     c = console_getkey ();
 #ifdef SUPPORT_SERIAL
   else if (terminal & TERMINAL_SERIAL)
@@ -1038,9 +1071,13 @@ checkkey (void)
 {
   int c = -1;
 
-  if (terminal & TERMINAL_CONSOLE)
+  if ((terminal & TERMINAL_CONSOLE)
+#ifdef SUPPORT_HERCULES
+      || (terminal & TERMINAL_HERCULES)
+#endif /* SUPPORT_HERCULES */
+      )
     c = console_checkkey ();
-
+  
 #ifdef SUPPORT_SERIAL
   if (terminal & TERMINAL_SERIAL)
     c = serial_checkkey ();
@@ -1090,6 +1127,11 @@ grub_putchar (int c)
   if (terminal & TERMINAL_CONSOLE)
     console_putchar (c);
 
+# ifdef SUPPORT_HERCULES
+  if (terminal & TERMINAL_HERCULES)
+    herc_putchar (c);
+# endif /* SUPPORT_HERCULES */
+
 # ifdef SUPPORT_SERIAL
   if (terminal & TERMINAL_SERIAL)
     serial_putchar (c);
@@ -1104,10 +1146,14 @@ gotoxy (int x, int y)
 {
   if (terminal & TERMINAL_CONSOLE)
     console_gotoxy (x, y);
+#ifdef SUPPORT_HERCULES
+  else if (terminal & TERMINAL_HERCULES)
+    herc_gotoxy (x, y);
+#endif /* SUPPORT_HERCULES */
 #ifdef SUPPORT_SERIAL
   else if (terminal & TERMINAL_SERIAL)
     serial_gotoxy (x, y);
-#endif
+#endif /* SUPPORT_SERIAL */
 }
 
 #ifdef SUPPORT_SERIAL
@@ -1126,10 +1172,14 @@ getxy (void)
   
   if (terminal & TERMINAL_CONSOLE)
     ret = console_getxy ();
+#ifdef SUPPORT_HERCULES
+  else if (terminal & TERMINAL_HERCULES)
+    ret = herc_getxy ();
+#endif /* SUPPORT_HERCULES */
 #ifdef SUPPORT_SERIAL
   else if (terminal & TERMINAL_SERIAL)
     ret = serial_getxy ();
-#endif
+#endif /* SUPPORT_SERIAL */
   
   return ret;
 }
@@ -1225,10 +1275,14 @@ cls (void)
 {
   if (terminal & TERMINAL_CONSOLE)
     console_cls ();
+#ifdef SUPPORT_HERCULES
+  else if (terminal & TERMINAL_HERCULES)
+    herc_cls ();
+#endif /* SUPPORT_HERCULES */
 #ifdef SUPPORT_SERIAL
   else if (terminal & TERMINAL_SERIAL)
     serial_cls ();
-#endif
+#endif /* SUPPORT_SERIAL */
 }
 
 #ifdef SUPPORT_SERIAL
@@ -1243,6 +1297,17 @@ serial_cls (void)
     grub_printf ("\e[H\e[J");
 }
 #endif /* SUPPORT_SERIAL */
+
+void
+set_attrib (int attr)
+{
+#ifdef SUPPORT_HERCULES
+  if (terminal & TERMINAL_HERCULES)
+    herc_set_attrib (attr);
+  else
+#endif /* SUPPORT_HERCULES */
+    console_set_attrib (attr);
+}
 #endif /* ! STAGE1_5 */
 
 int
@@ -1282,17 +1347,20 @@ nul_terminate (char *str)
 char *
 grub_strstr (const char *s1, const char *s2)
 {
-  const char *ptr, *tmp;
-
   while (*s1)
     {
+      const char *ptr, *tmp;
+
       ptr = s1;
       tmp = s2;
 
-      while (*s1 && *s1++ == *tmp++);
+      while (*tmp && *ptr == *tmp)
+	ptr++, tmp++;
+      
+      if (tmp > s2 && ! *tmp)
+	return (char *) s1;
 
-      if (tmp > s2 && !*(tmp - 1))
-	return (char *) ptr;
+      s1++;
     }
 
   return 0;
@@ -1386,7 +1454,6 @@ grub_memmove (void *to, const void *from, int len)
    return errnum ? NULL : to;
 }
 
-#ifndef STAGE1_5
 void *
 grub_memset (void *start, int c, int len)
 {
@@ -1401,6 +1468,7 @@ grub_memset (void *start, int c, int len)
   return errnum ? NULL : start;
 }
 
+#ifndef STAGE1_5
 char *
 grub_strcpy (char *dest, const char *src)
 {
