@@ -1,8 +1,7 @@
 /* cmdline.c - the device-independent GRUB text command line */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 1996  Erich Boleyn  <erich@uruk.org>
- *  Copyright (C) 1999, 2000  Free Software Foundation, Inc.
+ *  Copyright (C) 1999, 2000, 2001  Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +21,7 @@
 #include <shared.h>
 
 #ifdef SUPPORT_DISKLESS
+# define GRUB	1
 # include <etherboot.h>
 #endif
 
@@ -50,10 +50,9 @@ skip_to (int after_equal, char *cmdline)
 void
 print_cmdline_message (int forever)
 {
-  printf ("\
- [ Minimal BASH-like line editing is supported.  For the first word, TAB
-   lists possible command completions.  Anywhere else TAB lists the possible
-   completions of a device/filename.%s ]\n",
+  printf (" [ Minimal BASH-like line editing is supported.  For the first word, TAB\n"
+	  "   lists possible command completions.  Anywhere else TAB lists the possible\n"
+	  "   completions of a device/filename.%s ]\n",
 	  (forever ? "" : "  ESC at any time exits."));
 }
 
@@ -202,8 +201,6 @@ run_script (char *script, char *heap)
 	;
 
       grub_memmove (heap, old_entry, (int) cur_entry - (int) old_entry);
-      grub_printf ("%s\n", old_entry);
-
       if (! *heap)
 	{
 	  /* If there is no more command in SCRIPT...  */
@@ -219,7 +216,13 @@ run_script (char *script, char *heap)
       /* Find a builtin.  */
       builtin = find_command (heap);
       if (! builtin)
-	continue;
+	{
+	  grub_printf ("%s\n", old_entry);
+	  continue;
+	}
+
+      if (! (builtin->flags & BUILTIN_HIDDEN))
+	grub_printf ("%s\n", old_entry);
 
       /* If BUILTIN cannot be run in the command-line, skip it.  */
       if (! (builtin->flags & BUILTIN_CMDLINE))
