@@ -1,8 +1,7 @@
 /* disk_io.c - implement abstract BIOS disk input and output */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 1996  Erich Boleyn  <erich@uruk.org>
- *  Copyright (C) 1999, 2000  Free Software Foundation, Inc.
+ *  Copyright (C) 1999, 2000, 2001  Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,11 +23,15 @@
 
 #include "filesys.h"
 
-#ifndef STAGE1_5
+#ifdef GRUB_UTIL
+# include <device.h>
+#endif
+
 /* instrumentation variables */
 void (*disk_read_hook) (int, int, int) = NULL;
 void (*disk_read_func) (int, int, int) = NULL;
 
+#ifndef STAGE1_5
 int print_possibilities;
 
 static int do_completion;
@@ -55,6 +58,15 @@ struct fsys_entry fsys_table[NUM_FSYS + 1] =
 # endif
 # ifdef FSYS_REISERFS
   {"reiserfs", reiserfs_mount, reiserfs_read, reiserfs_dir, 0, reiserfs_embed},
+# endif
+# ifdef FSYS_VSTAFS
+  {"vstafs", vstafs_mount, vstafs_read, vstafs_dir, 0, 0},
+# endif
+# ifdef FSYS_JFS
+  {"jfs", jfs_mount, jfs_read, jfs_dir, 0, jfs_embed},
+# endif
+# ifdef FSYS_XFS
+  {"xfs", xfs_mount, xfs_read, xfs_dir, 0, 0},
 # endif
   /* XX FFS should come last as it's superblock is commonly crossing tracks
      on floppies from track 1 to 2, while others only use 1.  */
@@ -106,7 +118,6 @@ struct geometry buf_geom;
 int filepos;
 int filemax;
 
-
 int
 rawread (int drive, int sector, int byte_offset, int byte_len, char *buf)
 {
@@ -180,12 +191,32 @@ rawread (int drive, int sector, int byte_offset, int byte_len, char *buf)
 	    }
 	  else
 	    buf_track = track;
-	}
 
+	  if ((buf_track == 0 || sector == 0)
+	      && (PC_SLICE_TYPE (BUFFERADDR, 0) == PC_SLICE_TYPE_EZD
+		  || PC_SLICE_TYPE (BUFFERADDR, 1) == PC_SLICE_TYPE_EZD
+		  || PC_SLICE_TYPE (BUFFERADDR, 2) == PC_SLICE_TYPE_EZD
+		  || PC_SLICE_TYPE (BUFFERADDR, 3) == PC_SLICE_TYPE_EZD))
+	    {
+	      /* This is a EZD disk map sector 0 to sector 1 */
+	      if (buf_track == 0 || slen >= 2)
+		{
+		  /* We already read the sector 1, copy it to sector 0 */
+		  memmove ((char *) BUFFERADDR, 
+			   (char *) BUFFERADDR + SECTOR_SIZE, SECTOR_SIZE);
+		}
+	      else
+		{
+		  if (biosdisk (BIOSDISK_READ, drive, &buf_geom,
+				1, 1, BUFFERSEG))
+		    errnum = ERR_READ;
+		}
+	    }
+	}
+	  
       if (size > ((num_sect * SECTOR_SIZE) - byte_offset))
 	size = (num_sect * SECTOR_SIZE) - byte_offset;
 
-#ifndef STAGE1_5
       /*
        *  Instrumentation to tell which sectors were read and used.
        */
@@ -207,7 +238,7 @@ rawread (int drive, int sector, int byte_offset, int byte_len, char *buf)
 	      (*disk_read_func) (sector_num, 0, length);
 	    }
 	}
-#endif /* STAGE1_5 */
+
       memmove (buf, (char *) bufaddr, size);
 
       buf += size;
@@ -260,6 +291,68 @@ devread (int sector, int byte_offset, int byte_len, char *buf)
 }
 
 #ifndef STAGE1_5
+int
+rawwrite (int drive, int sector, char *buf)
+{
+  if (sector == 0)
+    {
+      if (biosdisk (BIOSDISK_READ, drive, &buf_geom, 0, 1, SCRATCHSEG))
+	{
+	  errnum = ERR_WRITE;
+	  return 0;
+	}
+
+      if (PC_SLICE_TYPE (SCRATCHADDR, 0) == PC_SLICE_TYPE_EZD
+	  || PC_SLICE_TYPE (SCRATCHADDR, 1) == PC_SLICE_TYPE_EZD
+	  || PC_SLICE_TYPE (SCRATCHADDR, 2) == PC_SLICE_TYPE_EZD
+	  || PC_SLICE_TYPE (SCRATCHADDR, 3) == PC_SLICE_TYPE_EZD)
+	sector = 1;
+    }
+  
+  memmove ((char *) SCRATCHADDR, buf, SECTOR_SIZE);
+  if (biosdisk (BIOSDISK_WRITE, drive, &buf_geom,
+		sector, 1, SCRATCHSEG))
+    {
+      errnum = ERR_WRITE;
+      return 0;
+    }
+
+  if (sector - sector % buf_geom.sectors == buf_track)
+    /* Clear the cache.  */
+    buf_track = -1;
+
+  return 1;
+}
+
+int
+devwrite (int sector, int sector_count, char *buf)
+{
+#if defined(GRUB_UTIL) && defined(__linux__)
+  if (current_partition != 0xFFFFFF)
+    {
+      /* If the grub shell is running under Linux and the user wants to
+	 embed a Stage 1.5 into a partition instead of a MBR, use system
+	 calls directly instead of biosdisk, because of the bug in
+	 Linux. *sigh*  */
+      return write_to_partition (device_map, current_drive, current_partition,
+				 sector, sector_count, buf);
+    }
+  else
+#endif /* GRUB_UTIL && __linux__ */
+    {
+      int i;
+      
+      for (i = 0; i < sector_count; i++)
+	{
+	  if (! rawwrite (current_drive, part_start + sector + i, 
+			  buf + (i << SECTOR_BITS)))
+	      return 0;
+
+	}
+      return 1;
+    }
+}
+
 static int
 sane_partition (void)
 {
@@ -302,12 +395,14 @@ attempt_mount (void)
 
 
 #ifndef STAGE1_5
-/* Turn on the active flag for the partition SAVED_PARATITION in the
+/* Turn on the active flag for the partition SAVED_PARTITION in the
    drive SAVED_DRIVE. If an error occurs, return zero, otherwise return
    non-zero.  */
 int
 make_saved_active (void)
 {
+  char mbr[512];
+
   if (saved_drive & 0x80)
     {
       /* Hard disk */
@@ -317,44 +412,37 @@ make_saved_active (void)
 	 meaningless. (XXX: Really?)  */
       if (part > 3)
 	{
-	  errnum = ERR_NO_PART;
+	  errnum = ERR_DEV_VALUES;
 	  return 0;
 	}
 
       /* Read the MBR in the scratch space.  */
-      if (! rawread (saved_drive, 0, 0, SECTOR_SIZE, (char *) SCRATCHADDR))
+      if (! rawread (saved_drive, 0, 0, SECTOR_SIZE, mbr))
 	return 0;
 
       /* If the partition is an extended partition, setting the active
 	 flag violates the specification by IBM.  */
-      if (IS_PC_SLICE_TYPE_EXTENDED (PC_SLICE_TYPE (SCRATCHADDR, part)))
+      if (IS_PC_SLICE_TYPE_EXTENDED (PC_SLICE_TYPE (mbr, part)))
 	{
 	  errnum = ERR_DEV_VALUES;
 	  return 0;
 	}
 
       /* Check if the active flag is disabled.  */
-      if (PC_SLICE_FLAG (SCRATCHADDR, part) != PC_SLICE_FLAG_BOOTABLE)
+      if (PC_SLICE_FLAG (mbr, part) != PC_SLICE_FLAG_BOOTABLE)
 	{
 	  int i;
 
 	  /* Clear all the active flags in this table.  */
 	  for (i = 0; i < 4; i++)
-	    PC_SLICE_FLAG (SCRATCHADDR, i) = 0;
+	    PC_SLICE_FLAG (mbr, i) = 0;
 
 	  /* Set the flag.  */
-	  PC_SLICE_FLAG (SCRATCHADDR, part) = PC_SLICE_FLAG_BOOTABLE;
-
-	  /* Clear the cache.  */
-	  buf_track = -1;
+	  PC_SLICE_FLAG (mbr, part) = PC_SLICE_FLAG_BOOTABLE;
 
 	  /* Write back the MBR.  */
-	  if (biosdisk (BIOSDISK_WRITE, saved_drive, &buf_geom,
-			0, 1, SCRATCHSEG))
-	    {
-	      errnum = ERR_WRITE;
-	      return 0;
-	    }
+	  if (! rawwrite (saved_drive, 0, mbr))
+	    return 0;
 	}
     }
   else
@@ -372,33 +460,49 @@ make_saved_active (void)
 int
 set_partition_hidden_flag (int hidden)
 {
-  if (current_drive & 0x80)
+  unsigned long part = 0xFFFFFF;
+  unsigned long start, len, offset, ext_offset;
+  int entry, type;
+  char mbr[512];
+  
+  /* The drive must be a hard disk.  */
+  if (! (current_drive & 0x80))
     {
-      int part = current_partition >> 16;
-
-      if (part > 3)
+      errnum = ERR_BAD_ARGUMENT;
+      return 1;
+    }
+  
+  /* The partition must be a PC slice.  */
+  if ((current_partition >> 16) == 0xFF
+      || (current_partition & 0xFFFF) != 0xFFFF)
+    {
+      errnum = ERR_BAD_ARGUMENT;
+      return 1;
+    }
+  
+  /* Look for the partition.  */
+  while (next_partition (current_drive, 0xFFFFFF, &part, &type,           
+			 &start, &len, &offset, &entry,
+			 &ext_offset, mbr))
+    {                                                                       
+      if (part == current_partition)
 	{
-          errnum = ERR_NO_PART;
-          return 0;
-        }
-
-      if (! rawread (current_drive, 0, 0, SECTOR_SIZE, (char *) SCRATCHADDR))
-        return 0;
-
-      if (hidden)
-	PC_SLICE_TYPE (SCRATCHADDR, part) |= PC_SLICE_TYPE_HIDDEN_FLAG;
-      else
-	PC_SLICE_TYPE (SCRATCHADDR, part) &= ~PC_SLICE_TYPE_HIDDEN_FLAG;
-      
-      buf_track = -1;
-      if (biosdisk (BIOSDISK_WRITE, current_drive, &buf_geom,
-		    0, 1, SCRATCHSEG))
-	{
-	  errnum = ERR_WRITE;
+	  /* Found.  */
+	  if (hidden)
+	    PC_SLICE_TYPE (mbr, entry) |= PC_SLICE_TYPE_HIDDEN_FLAG;
+	  else
+	    PC_SLICE_TYPE (mbr, entry) &= ~PC_SLICE_TYPE_HIDDEN_FLAG;       
+	  
+	  /* Write back the MBR to the disk.  */
+	  buf_track = -1;
+	  if (! rawwrite (current_drive, offset, mbr))
+	    return 1;
+	  
+	  /* Succeed.  */
 	  return 0;
 	}
     }
-
+  
   return 1;
 }
 
@@ -574,7 +678,7 @@ next_partition (unsigned long drive, unsigned long dest,
 
   /* If previous partition is a BSD partition or a PC slice which
      contains BSD partitions...  */
-  if ((*partition != 0xFFFFFF && IS_PC_SLICE_TYPE_BSD (*type))
+  if ((*partition != 0xFFFFFF && IS_PC_SLICE_TYPE_BSD (*type & 0xff))
       || ! (drive & 0x80))
     {
       if (*type == PC_SLICE_TYPE_NONE)
@@ -1499,17 +1603,13 @@ grub_read (char *buf, int len)
 	  if (size > len)
 	    size = len;
 
-#ifndef STAGE1_5
 	  disk_read_func = disk_read_hook;
-#endif /* STAGE1_5 */
 
 	  /* read current block and put it in the right place in memory */
 	  devread (BLK_BLKSTART (BLK_CUR_BLKLIST) + BLK_CUR_BLKNUM,
 		   off, size, buf);
 
-#ifndef STAGE1_5
 	  disk_read_func = NULL;
-#endif /* STAGE1_5 */
 
 	  len -= size;
 	  filepos += size;
